from cognite.powerops.client._generated.v1._api.alert import AlertAPI
from cognite.powerops.client._generated.v1._api.alert_query import AlertQueryAPI
from cognite.powerops.client._generated.v1._api.benchmarking_calculation_input import BenchmarkingCalculationInputAPI
from cognite.powerops.client._generated.v1._api.benchmarking_calculation_input_query import BenchmarkingCalculationInputQueryAPI
from cognite.powerops.client._generated.v1._api.benchmarking_calculation_input_shop_results import BenchmarkingCalculationInputShopResultsAPI
from cognite.powerops.client._generated.v1._api.benchmarking_calculation_output import BenchmarkingCalculationOutputAPI
from cognite.powerops.client._generated.v1._api.benchmarking_calculation_output_alerts import BenchmarkingCalculationOutputAlertsAPI
from cognite.powerops.client._generated.v1._api.benchmarking_calculation_output_benchmarking_results import BenchmarkingCalculationOutputBenchmarkingResultsAPI
from cognite.powerops.client._generated.v1._api.benchmarking_calculation_output_query import BenchmarkingCalculationOutputQueryAPI
from cognite.powerops.client._generated.v1._api.benchmarking_configuration_day_ahead import BenchmarkingConfigurationDayAheadAPI
from cognite.powerops.client._generated.v1._api.benchmarking_configuration_day_ahead_assets_per_shop_model import BenchmarkingConfigurationDayAheadAssetsPerShopModelAPI
from cognite.powerops.client._generated.v1._api.benchmarking_configuration_day_ahead_bid_configurations import BenchmarkingConfigurationDayAheadBidConfigurationsAPI
from cognite.powerops.client._generated.v1._api.benchmarking_configuration_day_ahead_query import BenchmarkingConfigurationDayAheadQueryAPI
from cognite.powerops.client._generated.v1._api.benchmarking_production_obligation_day_ahead import BenchmarkingProductionObligationDayAheadAPI
from cognite.powerops.client._generated.v1._api.benchmarking_production_obligation_day_ahead_query import BenchmarkingProductionObligationDayAheadQueryAPI
from cognite.powerops.client._generated.v1._api.benchmarking_production_obligation_day_ahead_time_series import BenchmarkingProductionObligationDayAheadTimeSeriesAPI
from cognite.powerops.client._generated.v1._api.benchmarking_result_day_ahead import BenchmarkingResultDayAheadAPI
from cognite.powerops.client._generated.v1._api.benchmarking_result_day_ahead_alerts import BenchmarkingResultDayAheadAlertsAPI
from cognite.powerops.client._generated.v1._api.benchmarking_result_day_ahead_query import BenchmarkingResultDayAheadQueryAPI
from cognite.powerops.client._generated.v1._api.benchmarking_shop_case import BenchmarkingShopCaseAPI
from cognite.powerops.client._generated.v1._api.benchmarking_shop_case_query import BenchmarkingShopCaseQueryAPI
from cognite.powerops.client._generated.v1._api.benchmarking_shop_case_shop_files import BenchmarkingShopCaseShopFilesAPI
from cognite.powerops.client._generated.v1._api.benchmarking_task_dispatcher_input_day_ahead import BenchmarkingTaskDispatcherInputDayAheadAPI
from cognite.powerops.client._generated.v1._api.benchmarking_task_dispatcher_input_day_ahead_query import BenchmarkingTaskDispatcherInputDayAheadQueryAPI
from cognite.powerops.client._generated.v1._api.benchmarking_task_dispatcher_output_day_ahead import BenchmarkingTaskDispatcherOutputDayAheadAPI
from cognite.powerops.client._generated.v1._api.benchmarking_task_dispatcher_output_day_ahead_alerts import BenchmarkingTaskDispatcherOutputDayAheadAlertsAPI
from cognite.powerops.client._generated.v1._api.benchmarking_task_dispatcher_output_day_ahead_benchmarking_sub_tasks import BenchmarkingTaskDispatcherOutputDayAheadBenchmarkingSubTasksAPI
from cognite.powerops.client._generated.v1._api.benchmarking_task_dispatcher_output_day_ahead_query import BenchmarkingTaskDispatcherOutputDayAheadQueryAPI
from cognite.powerops.client._generated.v1._api.bid_configuration_day_ahead import BidConfigurationDayAheadAPI
from cognite.powerops.client._generated.v1._api.bid_configuration_day_ahead_partials import BidConfigurationDayAheadPartialsAPI
from cognite.powerops.client._generated.v1._api.bid_configuration_day_ahead_query import BidConfigurationDayAheadQueryAPI
from cognite.powerops.client._generated.v1._api.bid_document_afrr import BidDocumentAFRRAPI
from cognite.powerops.client._generated.v1._api.bid_document_afrr_alerts import BidDocumentAFRRAlertsAPI
from cognite.powerops.client._generated.v1._api.bid_document_afrr_bids import BidDocumentAFRRBidsAPI
from cognite.powerops.client._generated.v1._api.bid_document_afrr_query import BidDocumentAFRRQueryAPI
from cognite.powerops.client._generated.v1._api.bid_document import BidDocumentAPI
from cognite.powerops.client._generated.v1._api.bid_document_alerts import BidDocumentAlertsAPI
from cognite.powerops.client._generated.v1._api.bid_document_day_ahead import BidDocumentDayAheadAPI
from cognite.powerops.client._generated.v1._api.bid_document_day_ahead_alerts import BidDocumentDayAheadAlertsAPI
from cognite.powerops.client._generated.v1._api.bid_document_day_ahead_partials import BidDocumentDayAheadPartialsAPI
from cognite.powerops.client._generated.v1._api.bid_document_day_ahead_query import BidDocumentDayAheadQueryAPI
from cognite.powerops.client._generated.v1._api.bid_document_query import BidDocumentQueryAPI
from cognite.powerops.client._generated.v1._api.bid_matrix import BidMatrixAPI
from cognite.powerops.client._generated.v1._api.bid_matrix_information import BidMatrixInformationAPI
from cognite.powerops.client._generated.v1._api.bid_matrix_information_alerts import BidMatrixInformationAlertsAPI
from cognite.powerops.client._generated.v1._api.bid_matrix_information_linked_time_series import BidMatrixInformationLinkedTimeSeriesAPI
from cognite.powerops.client._generated.v1._api.bid_matrix_information_query import BidMatrixInformationQueryAPI
from cognite.powerops.client._generated.v1._api.bid_matrix_information_underlying_bid_matrices import BidMatrixInformationUnderlyingBidMatricesAPI
from cognite.powerops.client._generated.v1._api.bid_matrix_query import BidMatrixQueryAPI
from cognite.powerops.client._generated.v1._api.bid_row import BidRowAPI
from cognite.powerops.client._generated.v1._api.bid_row_alerts import BidRowAlertsAPI
from cognite.powerops.client._generated.v1._api.bid_row_query import BidRowQueryAPI
from cognite.powerops.client._generated.v1._api.date_specification import DateSpecificationAPI
from cognite.powerops.client._generated.v1._api.date_specification_query import DateSpecificationQueryAPI
from cognite.powerops.client._generated.v1._api.function_input import FunctionInputAPI
from cognite.powerops.client._generated.v1._api.function_input_query import FunctionInputQueryAPI
from cognite.powerops.client._generated.v1._api.function_output import FunctionOutputAPI
from cognite.powerops.client._generated.v1._api.function_output_alerts import FunctionOutputAlertsAPI
from cognite.powerops.client._generated.v1._api.function_output_query import FunctionOutputQueryAPI
from cognite.powerops.client._generated.v1._api.generator import GeneratorAPI
from cognite.powerops.client._generated.v1._api.generator_availability_time_series import GeneratorAvailabilityTimeSeriesAPI
from cognite.powerops.client._generated.v1._api.generator_efficiency_curve import GeneratorEfficiencyCurveAPI
from cognite.powerops.client._generated.v1._api.generator_efficiency_curve_query import GeneratorEfficiencyCurveQueryAPI
from cognite.powerops.client._generated.v1._api.generator_query import GeneratorQueryAPI
from cognite.powerops.client._generated.v1._api.generator_start_stop_cost_time_series import GeneratorStartStopCostTimeSeriesAPI
from cognite.powerops.client._generated.v1._api.generator_turbine_efficiency_curves import GeneratorTurbineEfficiencyCurvesAPI
from cognite.powerops.client._generated.v1._api.market_configuration import MarketConfigurationAPI
from cognite.powerops.client._generated.v1._api.market_configuration_query import MarketConfigurationQueryAPI
from cognite.powerops.client._generated.v1._api.multi_scenario_partial_bid_matrix_calculation_input import MultiScenarioPartialBidMatrixCalculationInputAPI
from cognite.powerops.client._generated.v1._api.multi_scenario_partial_bid_matrix_calculation_input_price_production import MultiScenarioPartialBidMatrixCalculationInputPriceProductionAPI
from cognite.powerops.client._generated.v1._api.multi_scenario_partial_bid_matrix_calculation_input_query import MultiScenarioPartialBidMatrixCalculationInputQueryAPI
from cognite.powerops.client._generated.v1._api.partial_bid_configuration import PartialBidConfigurationAPI
from cognite.powerops.client._generated.v1._api.partial_bid_configuration_query import PartialBidConfigurationQueryAPI
from cognite.powerops.client._generated.v1._api.partial_bid_matrix_calculation_input import PartialBidMatrixCalculationInputAPI
from cognite.powerops.client._generated.v1._api.partial_bid_matrix_calculation_input_query import PartialBidMatrixCalculationInputQueryAPI
from cognite.powerops.client._generated.v1._api.partial_bid_matrix_calculation_output import PartialBidMatrixCalculationOutputAPI
from cognite.powerops.client._generated.v1._api.partial_bid_matrix_calculation_output_alerts import PartialBidMatrixCalculationOutputAlertsAPI
from cognite.powerops.client._generated.v1._api.partial_bid_matrix_calculation_output_query import PartialBidMatrixCalculationOutputQueryAPI
from cognite.powerops.client._generated.v1._api.partial_bid_matrix_information import PartialBidMatrixInformationAPI
from cognite.powerops.client._generated.v1._api.partial_bid_matrix_information_alerts import PartialBidMatrixInformationAlertsAPI
from cognite.powerops.client._generated.v1._api.partial_bid_matrix_information_linked_time_series import PartialBidMatrixInformationLinkedTimeSeriesAPI
from cognite.powerops.client._generated.v1._api.partial_bid_matrix_information_query import PartialBidMatrixInformationQueryAPI
from cognite.powerops.client._generated.v1._api.partial_bid_matrix_information_underlying_bid_matrices import PartialBidMatrixInformationUnderlyingBidMatricesAPI
from cognite.powerops.client._generated.v1._api.partial_bid_matrix_information_with_scenarios import PartialBidMatrixInformationWithScenariosAPI
from cognite.powerops.client._generated.v1._api.partial_bid_matrix_information_with_scenarios_alerts import PartialBidMatrixInformationWithScenariosAlertsAPI
from cognite.powerops.client._generated.v1._api.partial_bid_matrix_information_with_scenarios_linked_time_series import PartialBidMatrixInformationWithScenariosLinkedTimeSeriesAPI
from cognite.powerops.client._generated.v1._api.partial_bid_matrix_information_with_scenarios_multi_scenario_input import PartialBidMatrixInformationWithScenariosMultiScenarioInputAPI
from cognite.powerops.client._generated.v1._api.partial_bid_matrix_information_with_scenarios_query import PartialBidMatrixInformationWithScenariosQueryAPI
from cognite.powerops.client._generated.v1._api.partial_bid_matrix_information_with_scenarios_underlying_bid_matrices import PartialBidMatrixInformationWithScenariosUnderlyingBidMatricesAPI
from cognite.powerops.client._generated.v1._api.plant import PlantAPI
from cognite.powerops.client._generated.v1._api.plant_information import PlantInformationAPI
from cognite.powerops.client._generated.v1._api.plant_information_feeding_fee_time_series import PlantInformationFeedingFeeTimeSeriesAPI
from cognite.powerops.client._generated.v1._api.plant_information_generators import PlantInformationGeneratorsAPI
from cognite.powerops.client._generated.v1._api.plant_information_head_direct_time_series import PlantInformationHeadDirectTimeSeriesAPI
from cognite.powerops.client._generated.v1._api.plant_information_inlet_level_time_series import PlantInformationInletLevelTimeSeriesAPI
from cognite.powerops.client._generated.v1._api.plant_information_outlet_level_time_series import PlantInformationOutletLevelTimeSeriesAPI
from cognite.powerops.client._generated.v1._api.plant_information_production_max_time_series import PlantInformationProductionMaxTimeSeriesAPI
from cognite.powerops.client._generated.v1._api.plant_information_production_min_time_series import PlantInformationProductionMinTimeSeriesAPI
from cognite.powerops.client._generated.v1._api.plant_information_query import PlantInformationQueryAPI
from cognite.powerops.client._generated.v1._api.plant_information_water_value_time_series import PlantInformationWaterValueTimeSeriesAPI
from cognite.powerops.client._generated.v1._api.plant_query import PlantQueryAPI
from cognite.powerops.client._generated.v1._api.plant_water_value_based import PlantWaterValueBasedAPI
from cognite.powerops.client._generated.v1._api.plant_water_value_based_feeding_fee_time_series import PlantWaterValueBasedFeedingFeeTimeSeriesAPI
from cognite.powerops.client._generated.v1._api.plant_water_value_based_generators import PlantWaterValueBasedGeneratorsAPI
from cognite.powerops.client._generated.v1._api.plant_water_value_based_head_direct_time_series import PlantWaterValueBasedHeadDirectTimeSeriesAPI
from cognite.powerops.client._generated.v1._api.plant_water_value_based_inlet_level_time_series import PlantWaterValueBasedInletLevelTimeSeriesAPI
from cognite.powerops.client._generated.v1._api.plant_water_value_based_outlet_level_time_series import PlantWaterValueBasedOutletLevelTimeSeriesAPI
from cognite.powerops.client._generated.v1._api.plant_water_value_based_production_max_time_series import PlantWaterValueBasedProductionMaxTimeSeriesAPI
from cognite.powerops.client._generated.v1._api.plant_water_value_based_production_min_time_series import PlantWaterValueBasedProductionMinTimeSeriesAPI
from cognite.powerops.client._generated.v1._api.plant_water_value_based_query import PlantWaterValueBasedQueryAPI
from cognite.powerops.client._generated.v1._api.plant_water_value_based_water_value_time_series import PlantWaterValueBasedWaterValueTimeSeriesAPI
from cognite.powerops.client._generated.v1._api.power_asset import PowerAssetAPI
from cognite.powerops.client._generated.v1._api.power_asset_query import PowerAssetQueryAPI
from cognite.powerops.client._generated.v1._api.price_area_afrr import PriceAreaAFRRAPI
from cognite.powerops.client._generated.v1._api.price_area_afrr_activation_price_down import PriceAreaAFRRActivationPriceDownAPI
from cognite.powerops.client._generated.v1._api.price_area_afrr_activation_price_up import PriceAreaAFRRActivationPriceUpAPI
from cognite.powerops.client._generated.v1._api.price_area_afrr_capacity_price_down import PriceAreaAFRRCapacityPriceDownAPI
from cognite.powerops.client._generated.v1._api.price_area_afrr_capacity_price_up import PriceAreaAFRRCapacityPriceUpAPI
from cognite.powerops.client._generated.v1._api.price_area_afrr_own_capacity_allocation_down import PriceAreaAFRROwnCapacityAllocationDownAPI
from cognite.powerops.client._generated.v1._api.price_area_afrr_own_capacity_allocation_up import PriceAreaAFRROwnCapacityAllocationUpAPI
from cognite.powerops.client._generated.v1._api.price_area_afrr_query import PriceAreaAFRRQueryAPI
from cognite.powerops.client._generated.v1._api.price_area_afrr_relative_activation import PriceAreaAFRRRelativeActivationAPI
from cognite.powerops.client._generated.v1._api.price_area_afrr_total_capacity_allocation_down import PriceAreaAFRRTotalCapacityAllocationDownAPI
from cognite.powerops.client._generated.v1._api.price_area_afrr_total_capacity_allocation_up import PriceAreaAFRRTotalCapacityAllocationUpAPI
from cognite.powerops.client._generated.v1._api.price_area import PriceAreaAPI
from cognite.powerops.client._generated.v1._api.price_area_day_ahead import PriceAreaDayAheadAPI
from cognite.powerops.client._generated.v1._api.price_area_day_ahead_main_price_scenario import PriceAreaDayAheadMainPriceScenarioAPI
from cognite.powerops.client._generated.v1._api.price_area_day_ahead_price_scenarios import PriceAreaDayAheadPriceScenariosAPI
from cognite.powerops.client._generated.v1._api.price_area_day_ahead_query import PriceAreaDayAheadQueryAPI
from cognite.powerops.client._generated.v1._api.price_area_information import PriceAreaInformationAPI
from cognite.powerops.client._generated.v1._api.price_area_information_activation_price_down import PriceAreaInformationActivationPriceDownAPI
from cognite.powerops.client._generated.v1._api.price_area_information_activation_price_up import PriceAreaInformationActivationPriceUpAPI
from cognite.powerops.client._generated.v1._api.price_area_information_capacity_price_down import PriceAreaInformationCapacityPriceDownAPI
from cognite.powerops.client._generated.v1._api.price_area_information_capacity_price_up import PriceAreaInformationCapacityPriceUpAPI
from cognite.powerops.client._generated.v1._api.price_area_information_main_price_scenario import PriceAreaInformationMainPriceScenarioAPI
from cognite.powerops.client._generated.v1._api.price_area_information_own_capacity_allocation_down import PriceAreaInformationOwnCapacityAllocationDownAPI
from cognite.powerops.client._generated.v1._api.price_area_information_own_capacity_allocation_up import PriceAreaInformationOwnCapacityAllocationUpAPI
from cognite.powerops.client._generated.v1._api.price_area_information_price_scenarios import PriceAreaInformationPriceScenariosAPI
from cognite.powerops.client._generated.v1._api.price_area_information_query import PriceAreaInformationQueryAPI
from cognite.powerops.client._generated.v1._api.price_area_information_relative_activation import PriceAreaInformationRelativeActivationAPI
from cognite.powerops.client._generated.v1._api.price_area_information_total_capacity_allocation_down import PriceAreaInformationTotalCapacityAllocationDownAPI
from cognite.powerops.client._generated.v1._api.price_area_information_total_capacity_allocation_up import PriceAreaInformationTotalCapacityAllocationUpAPI
from cognite.powerops.client._generated.v1._api.price_area_query import PriceAreaQueryAPI
from cognite.powerops.client._generated.v1._api.price_production import PriceProductionAPI
from cognite.powerops.client._generated.v1._api.price_production_price import PriceProductionPriceAPI
from cognite.powerops.client._generated.v1._api.price_production_production import PriceProductionProductionAPI
from cognite.powerops.client._generated.v1._api.price_production_query import PriceProductionQueryAPI
from cognite.powerops.client._generated.v1._api.shop_attribute_mapping import ShopAttributeMappingAPI
from cognite.powerops.client._generated.v1._api.shop_attribute_mapping_query import ShopAttributeMappingQueryAPI
from cognite.powerops.client._generated.v1._api.shop_attribute_mapping_time_series import ShopAttributeMappingTimeSeriesAPI
from cognite.powerops.client._generated.v1._api.shop_based_partial_bid_configuration import ShopBasedPartialBidConfigurationAPI
from cognite.powerops.client._generated.v1._api.shop_based_partial_bid_configuration_query import ShopBasedPartialBidConfigurationQueryAPI
from cognite.powerops.client._generated.v1._api.shop_case import ShopCaseAPI
from cognite.powerops.client._generated.v1._api.shop_case_query import ShopCaseQueryAPI
from cognite.powerops.client._generated.v1._api.shop_case_shop_files import ShopCaseShopFilesAPI
from cognite.powerops.client._generated.v1._api.shop_commands import ShopCommandsAPI
from cognite.powerops.client._generated.v1._api.shop_commands_query import ShopCommandsQueryAPI
from cognite.powerops.client._generated.v1._api.shop_file import ShopFileAPI
from cognite.powerops.client._generated.v1._api.shop_file_query import ShopFileQueryAPI
from cognite.powerops.client._generated.v1._api.shop_model import ShopModelAPI
from cognite.powerops.client._generated.v1._api.shop_model_base_attribute_mappings import ShopModelBaseAttributeMappingsAPI
from cognite.powerops.client._generated.v1._api.shop_model_cog_shop_files_config import ShopModelCogShopFilesConfigAPI
from cognite.powerops.client._generated.v1._api.shop_model_query import ShopModelQueryAPI
from cognite.powerops.client._generated.v1._api.shop_model_with_assets import ShopModelWithAssetsAPI
from cognite.powerops.client._generated.v1._api.shop_model_with_assets_power_assets import ShopModelWithAssetsPowerAssetsAPI
from cognite.powerops.client._generated.v1._api.shop_model_with_assets_production_obligations import ShopModelWithAssetsProductionObligationsAPI
from cognite.powerops.client._generated.v1._api.shop_model_with_assets_query import ShopModelWithAssetsQueryAPI
from cognite.powerops.client._generated.v1._api.shop_output_time_series_definition import ShopOutputTimeSeriesDefinitionAPI
from cognite.powerops.client._generated.v1._api.shop_output_time_series_definition_query import ShopOutputTimeSeriesDefinitionQueryAPI
from cognite.powerops.client._generated.v1._api.shop_penalty_report import ShopPenaltyReportAPI
from cognite.powerops.client._generated.v1._api.shop_penalty_report_query import ShopPenaltyReportQueryAPI
from cognite.powerops.client._generated.v1._api.shop_preprocessor_input import ShopPreprocessorInputAPI
from cognite.powerops.client._generated.v1._api.shop_preprocessor_input_query import ShopPreprocessorInputQueryAPI
from cognite.powerops.client._generated.v1._api.shop_preprocessor_output import ShopPreprocessorOutputAPI
from cognite.powerops.client._generated.v1._api.shop_preprocessor_output_alerts import ShopPreprocessorOutputAlertsAPI
from cognite.powerops.client._generated.v1._api.shop_preprocessor_output_query import ShopPreprocessorOutputQueryAPI
from cognite.powerops.client._generated.v1._api.shop_result import ShopResultAPI
from cognite.powerops.client._generated.v1._api.shop_result_alerts import ShopResultAlertsAPI
from cognite.powerops.client._generated.v1._api.shop_result_output_time_series import ShopResultOutputTimeSeriesAPI
from cognite.powerops.client._generated.v1._api.shop_result_query import ShopResultQueryAPI
from cognite.powerops.client._generated.v1._api.shop_scenario import ShopScenarioAPI
from cognite.powerops.client._generated.v1._api.shop_scenario_attribute_mappings_override import ShopScenarioAttributeMappingsOverrideAPI
from cognite.powerops.client._generated.v1._api.shop_scenario_output_definition import ShopScenarioOutputDefinitionAPI
from cognite.powerops.client._generated.v1._api.shop_scenario_query import ShopScenarioQueryAPI
from cognite.powerops.client._generated.v1._api.shop_scenario_set import ShopScenarioSetAPI
from cognite.powerops.client._generated.v1._api.shop_scenario_set_query import ShopScenarioSetQueryAPI
from cognite.powerops.client._generated.v1._api.shop_scenario_set_scenarios import ShopScenarioSetScenariosAPI
from cognite.powerops.client._generated.v1._api.shop_time_resolution import ShopTimeResolutionAPI
from cognite.powerops.client._generated.v1._api.shop_time_resolution_query import ShopTimeResolutionQueryAPI
from cognite.powerops.client._generated.v1._api.shop_time_series import ShopTimeSeriesAPI
from cognite.powerops.client._generated.v1._api.shop_time_series_query import ShopTimeSeriesQueryAPI
from cognite.powerops.client._generated.v1._api.shop_time_series_time_series import ShopTimeSeriesTimeSeriesAPI
from cognite.powerops.client._generated.v1._api.shop_trigger_input import ShopTriggerInputAPI
from cognite.powerops.client._generated.v1._api.shop_trigger_input_query import ShopTriggerInputQueryAPI
from cognite.powerops.client._generated.v1._api.shop_trigger_output import ShopTriggerOutputAPI
from cognite.powerops.client._generated.v1._api.shop_trigger_output_alerts import ShopTriggerOutputAlertsAPI
from cognite.powerops.client._generated.v1._api.shop_trigger_output_query import ShopTriggerOutputQueryAPI
from cognite.powerops.client._generated.v1._api.task_dispatcher_input import TaskDispatcherInputAPI
from cognite.powerops.client._generated.v1._api.task_dispatcher_input_query import TaskDispatcherInputQueryAPI
from cognite.powerops.client._generated.v1._api.task_dispatcher_output import TaskDispatcherOutputAPI
from cognite.powerops.client._generated.v1._api.task_dispatcher_output_alerts import TaskDispatcherOutputAlertsAPI
from cognite.powerops.client._generated.v1._api.task_dispatcher_output_process_sub_tasks import TaskDispatcherOutputProcessSubTasksAPI
from cognite.powerops.client._generated.v1._api.task_dispatcher_output_query import TaskDispatcherOutputQueryAPI
from cognite.powerops.client._generated.v1._api.total_bid_matrix_calculation_input import TotalBidMatrixCalculationInputAPI
from cognite.powerops.client._generated.v1._api.total_bid_matrix_calculation_input_partial_bid_matrices import TotalBidMatrixCalculationInputPartialBidMatricesAPI
from cognite.powerops.client._generated.v1._api.total_bid_matrix_calculation_input_query import TotalBidMatrixCalculationInputQueryAPI
from cognite.powerops.client._generated.v1._api.total_bid_matrix_calculation_output import TotalBidMatrixCalculationOutputAPI
from cognite.powerops.client._generated.v1._api.total_bid_matrix_calculation_output_alerts import TotalBidMatrixCalculationOutputAlertsAPI
from cognite.powerops.client._generated.v1._api.total_bid_matrix_calculation_output_query import TotalBidMatrixCalculationOutputQueryAPI
from cognite.powerops.client._generated.v1._api.turbine_efficiency_curve import TurbineEfficiencyCurveAPI
from cognite.powerops.client._generated.v1._api.turbine_efficiency_curve_query import TurbineEfficiencyCurveQueryAPI
from cognite.powerops.client._generated.v1._api.water_value_based_partial_bid_configuration import WaterValueBasedPartialBidConfigurationAPI
from cognite.powerops.client._generated.v1._api.water_value_based_partial_bid_configuration_query import WaterValueBasedPartialBidConfigurationQueryAPI
from cognite.powerops.client._generated.v1._api.water_value_based_partial_bid_matrix_calculation_input import WaterValueBasedPartialBidMatrixCalculationInputAPI
from cognite.powerops.client._generated.v1._api.water_value_based_partial_bid_matrix_calculation_input_query import WaterValueBasedPartialBidMatrixCalculationInputQueryAPI
from cognite.powerops.client._generated.v1._api.watercourse import WatercourseAPI
from cognite.powerops.client._generated.v1._api.watercourse_query import WatercourseQueryAPI

__all__ = [
    "AlertAPI",
    "AlertQueryAPI",
    "BenchmarkingCalculationInputAPI",
    "BenchmarkingCalculationInputQueryAPI",
    "BenchmarkingCalculationInputShopResultsAPI",
    "BenchmarkingCalculationOutputAPI",
    "BenchmarkingCalculationOutputAlertsAPI",
    "BenchmarkingCalculationOutputBenchmarkingResultsAPI",
    "BenchmarkingCalculationOutputQueryAPI",
    "BenchmarkingConfigurationDayAheadAPI",
    "BenchmarkingConfigurationDayAheadAssetsPerShopModelAPI",
    "BenchmarkingConfigurationDayAheadBidConfigurationsAPI",
    "BenchmarkingConfigurationDayAheadQueryAPI",
    "BenchmarkingProductionObligationDayAheadAPI",
    "BenchmarkingProductionObligationDayAheadQueryAPI",
    "BenchmarkingProductionObligationDayAheadTimeSeriesAPI",
    "BenchmarkingResultDayAheadAPI",
    "BenchmarkingResultDayAheadAlertsAPI",
    "BenchmarkingResultDayAheadQueryAPI",
    "BenchmarkingShopCaseAPI",
    "BenchmarkingShopCaseQueryAPI",
    "BenchmarkingShopCaseShopFilesAPI",
    "BenchmarkingTaskDispatcherInputDayAheadAPI",
    "BenchmarkingTaskDispatcherInputDayAheadQueryAPI",
    "BenchmarkingTaskDispatcherOutputDayAheadAPI",
    "BenchmarkingTaskDispatcherOutputDayAheadAlertsAPI",
    "BenchmarkingTaskDispatcherOutputDayAheadBenchmarkingSubTasksAPI",
    "BenchmarkingTaskDispatcherOutputDayAheadQueryAPI",
    "BidConfigurationDayAheadAPI",
    "BidConfigurationDayAheadPartialsAPI",
    "BidConfigurationDayAheadQueryAPI",
    "BidDocumentAFRRAPI",
    "BidDocumentAFRRAlertsAPI",
    "BidDocumentAFRRBidsAPI",
    "BidDocumentAFRRQueryAPI",
    "BidDocumentAPI",
    "BidDocumentAlertsAPI",
    "BidDocumentDayAheadAPI",
    "BidDocumentDayAheadAlertsAPI",
    "BidDocumentDayAheadPartialsAPI",
    "BidDocumentDayAheadQueryAPI",
    "BidDocumentQueryAPI",
    "BidMatrixAPI",
    "BidMatrixInformationAPI",
    "BidMatrixInformationAlertsAPI",
    "BidMatrixInformationLinkedTimeSeriesAPI",
    "BidMatrixInformationQueryAPI",
    "BidMatrixInformationUnderlyingBidMatricesAPI",
    "BidMatrixQueryAPI",
    "BidRowAPI",
    "BidRowAlertsAPI",
    "BidRowQueryAPI",
    "DateSpecificationAPI",
    "DateSpecificationQueryAPI",
    "FunctionInputAPI",
    "FunctionInputQueryAPI",
    "FunctionOutputAPI",
    "FunctionOutputAlertsAPI",
    "FunctionOutputQueryAPI",
    "GeneratorAPI",
    "GeneratorAvailabilityTimeSeriesAPI",
    "GeneratorEfficiencyCurveAPI",
    "GeneratorEfficiencyCurveQueryAPI",
    "GeneratorQueryAPI",
    "GeneratorStartStopCostTimeSeriesAPI",
    "GeneratorTurbineEfficiencyCurvesAPI",
    "MarketConfigurationAPI",
    "MarketConfigurationQueryAPI",
    "MultiScenarioPartialBidMatrixCalculationInputAPI",
    "MultiScenarioPartialBidMatrixCalculationInputPriceProductionAPI",
    "MultiScenarioPartialBidMatrixCalculationInputQueryAPI",
    "PartialBidConfigurationAPI",
    "PartialBidConfigurationQueryAPI",
    "PartialBidMatrixCalculationInputAPI",
    "PartialBidMatrixCalculationInputQueryAPI",
    "PartialBidMatrixCalculationOutputAPI",
    "PartialBidMatrixCalculationOutputAlertsAPI",
    "PartialBidMatrixCalculationOutputQueryAPI",
    "PartialBidMatrixInformationAPI",
    "PartialBidMatrixInformationAlertsAPI",
    "PartialBidMatrixInformationLinkedTimeSeriesAPI",
    "PartialBidMatrixInformationQueryAPI",
    "PartialBidMatrixInformationUnderlyingBidMatricesAPI",
    "PartialBidMatrixInformationWithScenariosAPI",
    "PartialBidMatrixInformationWithScenariosAlertsAPI",
    "PartialBidMatrixInformationWithScenariosLinkedTimeSeriesAPI",
    "PartialBidMatrixInformationWithScenariosMultiScenarioInputAPI",
    "PartialBidMatrixInformationWithScenariosQueryAPI",
    "PartialBidMatrixInformationWithScenariosUnderlyingBidMatricesAPI",
    "PlantAPI",
    "PlantInformationAPI",
    "PlantInformationFeedingFeeTimeSeriesAPI",
    "PlantInformationGeneratorsAPI",
    "PlantInformationHeadDirectTimeSeriesAPI",
    "PlantInformationInletLevelTimeSeriesAPI",
    "PlantInformationOutletLevelTimeSeriesAPI",
    "PlantInformationProductionMaxTimeSeriesAPI",
    "PlantInformationProductionMinTimeSeriesAPI",
    "PlantInformationQueryAPI",
    "PlantInformationWaterValueTimeSeriesAPI",
    "PlantQueryAPI",
    "PlantWaterValueBasedAPI",
    "PlantWaterValueBasedFeedingFeeTimeSeriesAPI",
    "PlantWaterValueBasedGeneratorsAPI",
    "PlantWaterValueBasedHeadDirectTimeSeriesAPI",
    "PlantWaterValueBasedInletLevelTimeSeriesAPI",
    "PlantWaterValueBasedOutletLevelTimeSeriesAPI",
    "PlantWaterValueBasedProductionMaxTimeSeriesAPI",
    "PlantWaterValueBasedProductionMinTimeSeriesAPI",
    "PlantWaterValueBasedQueryAPI",
    "PlantWaterValueBasedWaterValueTimeSeriesAPI",
    "PowerAssetAPI",
    "PowerAssetQueryAPI",
    "PriceAreaAFRRAPI",
    "PriceAreaAFRRActivationPriceDownAPI",
    "PriceAreaAFRRActivationPriceUpAPI",
    "PriceAreaAFRRCapacityPriceDownAPI",
    "PriceAreaAFRRCapacityPriceUpAPI",
    "PriceAreaAFRROwnCapacityAllocationDownAPI",
    "PriceAreaAFRROwnCapacityAllocationUpAPI",
    "PriceAreaAFRRQueryAPI",
    "PriceAreaAFRRRelativeActivationAPI",
    "PriceAreaAFRRTotalCapacityAllocationDownAPI",
    "PriceAreaAFRRTotalCapacityAllocationUpAPI",
    "PriceAreaAPI",
    "PriceAreaDayAheadAPI",
    "PriceAreaDayAheadMainPriceScenarioAPI",
    "PriceAreaDayAheadPriceScenariosAPI",
    "PriceAreaDayAheadQueryAPI",
    "PriceAreaInformationAPI",
    "PriceAreaInformationActivationPriceDownAPI",
    "PriceAreaInformationActivationPriceUpAPI",
    "PriceAreaInformationCapacityPriceDownAPI",
    "PriceAreaInformationCapacityPriceUpAPI",
    "PriceAreaInformationMainPriceScenarioAPI",
    "PriceAreaInformationOwnCapacityAllocationDownAPI",
    "PriceAreaInformationOwnCapacityAllocationUpAPI",
    "PriceAreaInformationPriceScenariosAPI",
    "PriceAreaInformationQueryAPI",
    "PriceAreaInformationRelativeActivationAPI",
    "PriceAreaInformationTotalCapacityAllocationDownAPI",
    "PriceAreaInformationTotalCapacityAllocationUpAPI",
    "PriceAreaQueryAPI",
    "PriceProductionAPI",
    "PriceProductionPriceAPI",
    "PriceProductionProductionAPI",
    "PriceProductionQueryAPI",
    "ShopAttributeMappingAPI",
    "ShopAttributeMappingQueryAPI",
    "ShopAttributeMappingTimeSeriesAPI",
    "ShopBasedPartialBidConfigurationAPI",
    "ShopBasedPartialBidConfigurationQueryAPI",
    "ShopCaseAPI",
    "ShopCaseQueryAPI",
    "ShopCaseShopFilesAPI",
    "ShopCommandsAPI",
    "ShopCommandsQueryAPI",
    "ShopFileAPI",
    "ShopFileQueryAPI",
    "ShopModelAPI",
    "ShopModelBaseAttributeMappingsAPI",
    "ShopModelCogShopFilesConfigAPI",
    "ShopModelQueryAPI",
    "ShopModelWithAssetsAPI",
    "ShopModelWithAssetsPowerAssetsAPI",
    "ShopModelWithAssetsProductionObligationsAPI",
    "ShopModelWithAssetsQueryAPI",
    "ShopOutputTimeSeriesDefinitionAPI",
    "ShopOutputTimeSeriesDefinitionQueryAPI",
    "ShopPenaltyReportAPI",
    "ShopPenaltyReportQueryAPI",
    "ShopPreprocessorInputAPI",
    "ShopPreprocessorInputQueryAPI",
    "ShopPreprocessorOutputAPI",
    "ShopPreprocessorOutputAlertsAPI",
    "ShopPreprocessorOutputQueryAPI",
    "ShopResultAPI",
    "ShopResultAlertsAPI",
    "ShopResultOutputTimeSeriesAPI",
    "ShopResultQueryAPI",
    "ShopScenarioAPI",
    "ShopScenarioAttributeMappingsOverrideAPI",
    "ShopScenarioOutputDefinitionAPI",
    "ShopScenarioQueryAPI",
    "ShopScenarioSetAPI",
    "ShopScenarioSetQueryAPI",
    "ShopScenarioSetScenariosAPI",
    "ShopTimeResolutionAPI",
    "ShopTimeResolutionQueryAPI",
    "ShopTimeSeriesAPI",
    "ShopTimeSeriesQueryAPI",
    "ShopTimeSeriesTimeSeriesAPI",
    "ShopTriggerInputAPI",
    "ShopTriggerInputQueryAPI",
    "ShopTriggerOutputAPI",
    "ShopTriggerOutputAlertsAPI",
    "ShopTriggerOutputQueryAPI",
    "TaskDispatcherInputAPI",
    "TaskDispatcherInputQueryAPI",
    "TaskDispatcherOutputAPI",
    "TaskDispatcherOutputAlertsAPI",
    "TaskDispatcherOutputProcessSubTasksAPI",
    "TaskDispatcherOutputQueryAPI",
    "TotalBidMatrixCalculationInputAPI",
    "TotalBidMatrixCalculationInputPartialBidMatricesAPI",
    "TotalBidMatrixCalculationInputQueryAPI",
    "TotalBidMatrixCalculationOutputAPI",
    "TotalBidMatrixCalculationOutputAlertsAPI",
    "TotalBidMatrixCalculationOutputQueryAPI",
    "TurbineEfficiencyCurveAPI",
    "TurbineEfficiencyCurveQueryAPI",
    "WaterValueBasedPartialBidConfigurationAPI",
    "WaterValueBasedPartialBidConfigurationQueryAPI",
    "WaterValueBasedPartialBidMatrixCalculationInputAPI",
    "WaterValueBasedPartialBidMatrixCalculationInputQueryAPI",
    "WatercourseAPI",
    "WatercourseQueryAPI",
]
